// Load data from JSON file
d3.json('data.json')
  .then((data) => {
    const svg = d3.select('svg');
    const width = +svg.attr('width');
    const height = +svg.attr('height');

    // Create simulation
    const simulation = d3
      .forceSimulation(data.nodes)
      .force(
        'link',
        d3
          .forceLink(data.links)
          .id((d) => d.id)
          .distance((d) => d.distance * 0.25),
      )
      .force('charge', d3.forceManyBody().strength(-100))
      .force(
        'center',
        d3.forceCenter(width / 2, height / 2),
      );

    // Create links
    const links = svg
      .selectAll('.link')
      .data(data.links)
      .enter()
      .append('line')
      .attr('class', 'link')
      .style('stroke', '#999')
      .style('stroke-width', 2); // Set a constant stroke-width for links

    // Create text elements for displaying distance
    const linkLabels = svg
      .selectAll('.link-label')
      .data(data.links)
      .enter()
      .append('text')
      .attr('class', 'link-label')
      .text((d) => `${d.distance} miles`)
      .style('font-size', '10px')
      .attr('text-anchor', 'middle');

    // Create nodes
    const nodes = svg
      .selectAll('.node')
      .data(data.nodes)
      .enter()
      .append('g')
      .attr('class', 'node');

    // Append circles to nodes
    nodes
      .append('circle')
      .attr('r', 10)
      .attr('fill', '#1f77b4');

    // Append text labels to nodes
    nodes
      .append('text')
      .text((d) => d.id)
      .attr('dy', 4)
      .style('font-size', '12px')
      .attr('text-anchor', 'middle');

    // Tick function to update positions
    simulation.on('tick', () => {
      links
        .attr('x1', (d) => d.source.x)
        .attr('y1', (d) => d.source.y)
        .attr('x2', (d) => d.target.x)
        .attr('y2', (d) => d.target.y);

      linkLabels
        .attr('x', (d) => (d.source.x + d.target.x) / 2)
        .attr('y', (d) => (d.source.y + d.target.y) / 2)
        .each(function (d) {
          // Adjust position to prevent overlap
          const label = d3.select(this);
          const bbox = label.node().getBBox();
          const x =
            (d.source.x + d.target.x) / 2 - bbox.width / 2;
          const y =
            (d.source.y + d.target.y) / 2 + bbox.height / 2;
          label.attr('x', x).attr('y', y);
        });

      nodes.attr(
        'transform',
        (d) => `translate(${d.x},${d.y})`,
      );
    });
  })
  .catch((error) => {
    console.error('Error loading data:', error);
  });
